@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.1.1   2012/05/05
+ * Version: 0.1.5   2012/08/31
  */
 
 #include "internal.h"
@@ -599,7 +599,9 @@ void ccs_fill_path_info(struct ccs_path_info *ptr)
 char *ccs_get_exe(void)
 {
 	struct mm_struct *mm;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 7, 0)
 	struct vm_area_struct *vma;
+#endif
 	bool done = false;
 	char *cp = NULL;
 	if (current->flags & PF_KTHREAD)
@@ -608,6 +610,12 @@ char *ccs_get_exe(void)
 	if (!mm)
 		goto task_has_no_mm;
 	down_read(&mm->mmap_sem);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 7, 0)
+	if (mm->exe_file) {
+		cp = ccs_realpath(&mm->exe_file->f_path);
+		done = true;
+	}
+#else
 	for (vma = mm->mmap; vma; vma = vma->vm_next) {
 		if ((vma->vm_flags & VM_EXECUTABLE) && vma->vm_file) {
 			cp = ccs_realpath(&vma->vm_file->f_path);
@@ -615,6 +623,7 @@ char *ccs_get_exe(void)
 			break;
 		}
 	}
+#endif
 	up_read(&mm->mmap_sem);
 	if (done)
 		return cp;
