@@ -33,25 +33,28 @@ int search_binary_handler(struct linux_binprm *bprm, struct pt_regs *regs);
 
 #ifdef CONFIG_CAITSITH
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
-/* Obtain prototype of __d_path() or d_absolute_path(). */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36) && LINUX_VERSION_CODE < KERNEL_VERSION(3, 2, 0)
+/* Obtain prototype of __d_path(). */
 #include <linux/dcache.h>
 #endif
-/* Obtain prototype of find_task_by_vpid() and find_task_by_pid_ns(). */
-#include <linux/sched.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
+/* Obtain definition of kuid_t and kgid_t. */
+#include <linux/uidgid.h>
+#endif
 
 /* For exporting variables and functions. */
 struct caitsith_exports {
 	void (*load_policy) (const char *filename);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 2, 0)
-	typeof(d_absolute_path) (*d_absolute_path);
+	char * (*d_absolute_path) (const struct path *, char *, int);
 #elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
 	typeof(__d_path) (*__d_path);
 #else
 	spinlock_t *vfsmount_lock;
 #endif
-	typeof(find_task_by_vpid) (*find_task_by_vpid);
-	typeof(find_task_by_pid_ns) (*find_task_by_pid_ns);
+	struct task_struct * (*find_task_by_vpid) (pid_t nr);
+	struct task_struct * (*find_task_by_pid_ns) (pid_t nr,
+						     struct pid_namespace *ns);
 };
 
 /* For doing access control. */
@@ -60,7 +63,7 @@ struct caitsith_operations {
 	int (*chroot_permission) (struct path *path);
 	int (*pivot_root_permission) (struct path *old_path,
 				      struct path *new_path);
-	int (*mount_permission) (char *dev_name, struct path *path,
+	int (*mount_permission) (const char *dev_name, struct path *path,
 				 const char *type, unsigned long flags,
 				 void *data_page);
 	int (*umount_permission) (struct vfsmount *mnt, int flags);
@@ -155,14 +158,13 @@ static inline int ccs_pivot_root_permission(struct path *old_path,
 	return func ? func(old_path, new_path) : 0;
 }
 
-static inline int ccs_mount_permission(char *dev_name, struct path *path,
-				       char *type, unsigned long flags,
+static inline int ccs_mount_permission(const char *dev_name, struct path *path,
+				       const char *type, unsigned long flags,
 				       void *data_page)
 {
-	int (*func) (char *, struct path *, const char *, unsigned long,
+	int (*func) (const char *, struct path *, const char *, unsigned long,
 		     void *) = caitsith_ops.mount_permission;
-	return func ? func(dev_name, path, (const char *) type, flags,
-			   data_page) : 0;
+	return func ? func(dev_name, path, type, flags, data_page) : 0;
 }
 
 static inline int ccs_umount_permission(struct vfsmount *mnt, int flags)
@@ -370,8 +372,8 @@ static inline int ccs_pivot_root_permission(struct path *old_path,
 	return 0;
 }
 
-static inline int ccs_mount_permission(char *dev_name, struct path *path,
-				       char *type, unsigned long flags,
+static inline int ccs_mount_permission(const char *dev_name, struct path *path,
+				       const char *type, unsigned long flags,
 				       void *data_page)
 {
 	return 0;
